Attribute VB_Name = "mPlotText"
'----------------------------------------------------------------------------------------------------------------------------
'
'   PlotText() subroutine draws rotated text and allows the use of any raster operation (ROP)
'
'   I created this as a way to draw a mixture of graphics and text, using an Xor draw mode.
'
'   by Stephen Redmond steveredmond@sympatico.ca
'
'   November 2004
'
'----------------------------------------------------------------------------------------------------------------------------


Option Explicit

Private Const ANSI_CHARSET = 0                                              ' ANSI character set
Private Const CLIP_CHARACTER_PRECIS = 1                                     ' clip one character at a time, OR'd
Private Const CLIP_LH_ANGLES = 16                                           ' Needed for tilted fonts, OR'd
Private Const FONT_CLIP_PRECISION = CLIP_LH_ANGLES
Private Const FONT_ITALIC = 0                                               ' Italics T/F
Private Const FONT_NAME = "Arial"                                           ' typeface/font name
Private Const FONT_OUTPUT_PRECISION = 0                                     ' don't care
Private Const FONT_ORIENTATION = 0                                          ' don't care, Windows assumes same as escapement
Private Const FONT_PITCH_N_FAMILY = 0                                       ' don't care, let font mapper pick easily
Private Const FONT_QUALITY = 0                                              ' don't care, let font mapper pick easily
Private Const FONT_STRIKEOUT = 0                                            ' Strikeout T/F
Private Const FONT_UNDERLINE = 0                                            ' Underline T/F
Private Const FW_BOLD = 700                                                 ' Bold weight
Private Const FW_DONTCARE = 0                                               ' don't care, let font mapper use default weight
Private Const FW_NORMAL = 400                                               ' Normal weight
Private Const FONT_WEIGHT = FW_NORMAL
Private Const FONT_WIDTH = 0                                                ' don't care, let font mapper pick according to height
Private Const TRANSPARENT = 1                                               ' background mode
Private Declare Function BeginPath Lib "gdi32" (ByVal hdc As Long) As Long
Private Declare Function CreateFont Lib "gdi32" Alias "CreateFontA" (ByVal H As Long, ByVal W As Long, ByVal e As Long, ByVal o As Long, ByVal W As Long, ByVal i As Long, ByVal U As Long, ByVal S As Long, ByVal c As Long, ByVal OP As Long, ByVal cp As Long, ByVal Q As Long, ByVal PAF As Long, ByVal f As String) As Long
Private Declare Function CreateSolidBrush Lib "gdi32" (ByVal crColor As Long) As Long
Private Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long
Private Declare Function EndPath Lib "gdi32" (ByVal hdc As Long) As Long
Private Declare Function FillPath Lib "gdi32" (ByVal hdc As Long) As Long
Private Declare Function RestoreDC Lib "gdi32" (ByVal hdc As Long, ByVal nSavedDC As Long) As Long
Private Declare Function SetBkMode Lib "gdi32" (ByVal hdc As Long, ByVal nBkMode As Long) As Long
Private Declare Function SetROP2 Lib "gdi32" (ByVal hdc As Long, ByVal nDrawMode As Long) As Long
Private Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long
Private Declare Function SaveDC Lib "gdi32" (ByVal hdc As Long) As Long
Private Declare Function TextOut Lib "gdi32" Alias "TextOutA" (ByVal hdc As Long, ByVal X As Long, ByVal Y As Long, ByVal lpString As String, ByVal nCount As Long) As Long


'
' Obj is a picture object, such as a PictureBox or Form
' X and Y are the starting position of the text in Pixels
' Angle is in degrees of counterclockwise rotation, range 0 - 359.9 degrees
' FontSize is the character height in Pixels
' Txt is the text string to output
' DrawMode is the Raster Operation, e.g., vbCopyPen, vbNotXorPen
' Color s the text color, anything but vbWhite
' Many more font attributes could be variables, replacing constants in CreateFont() below
'
Public Sub PlotText(Obj As Object, _
                    ByRef X As Single, _
                    ByRef Y As Single, _
                    ByRef Angle As Single, _
                    ByRef Fontsize As Long, _
                    ByVal Txt As String, _
                    ByVal DrawMode As DrawModeConstants, _
                    ByVal color As Long _
                    )
    Dim savedDC As Long
    Dim Langle As Long
    Dim new_font As Long, old_font As Long
    Dim dl As Long                                                          'dummy to receive API returns
    Dim iROP2 As Long
    Dim usebrush As Long, oldbrush As Long
    
    If Fontsize < 0 Then Exit Sub                                           'guard against nonsense
    Langle = CLng(Angle * 10) Mod 3600                                      'convert angle to a Long in tenths of a degree
    savedDC = SaveDC(Obj.hdc)                                               'save Device Context
    
    new_font = CreateFont(Fontsize, FONT_WIDTH, _
        Langle, FONT_ORIENTATION, _
        FONT_WEIGHT, FONT_ITALIC, FONT_UNDERLINE, FONT_STRIKEOUT, _
        ANSI_CHARSET, FONT_OUTPUT_PRECISION, FONT_CLIP_PRECISION, _
        FONT_QUALITY, FONT_PITCH_N_FAMILY, FONT_NAME)                       'create the rotated font
    old_font = SelectObject(Obj.hdc, new_font)                              'select the new font, and save the old font (Important)!
    'set background mode to draw text with a transparent background
    '...obj.FontTransparent does NOT work for this
    dl = SetBkMode(Obj.hdc, TRANSPARENT)
    dl = BeginPath&(Obj.hdc)                                                'start saving graphics output into a Path
    dl = TextOut(Obj.hdc, X, Y, Txt, Len(Txt))                              'draw the text into the Path ... obj.Print does NOT work for this
    dl = EndPath(Obj.hdc)                                                   'finish saving to the Path
    dl = SelectObject(Obj.hdc, old_font)                                    'restore the original font
    dl = DeleteObject(new_font)                                             'free font resources (important!)
    '... if we wanted to transform the Path points we could do it here
    '... see MSDN for the demo C program TEXTFX.EXE
    'now draw and fill the saved path
    '... we don't get any outlines with just FillPath()
    iROP2 = SetROP2(Obj.hdc, DrawMode)                                      'set the drawmode, e.g., R2_NOTXORPEN/vbNotXorPen for Xor drawing
    usebrush = CreateSolidBrush(color)                                      'set the text color, create a brush to for the text areas
    oldbrush = SelectObject(Obj.hdc, usebrush)                              'select the new brush, and save the old brush (Important!)
    dl = FillPath(Obj.hdc)                                                  'draw the text saved in the Path, without outlines
    dl = SetROP2(Obj.hdc, iROP2)                                            'restore the original draw mode
    dl = SelectObject(Obj.hdc, oldbrush)                                    'restore the original Brush
    dl = DeleteObject(usebrush)                                             'free Brush resources (Important!)
    dl = RestoreDC(Obj.hdc, savedDC)                                        'restore the original Device Context (Important)
End Sub


